﻿using TotalSynergyAPIService.Domain;
using TotalSynergyAPIService.Domain.Services;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;
using TotalSynergyAPIService.Domain.Models;
using Newtonsoft.Json;

namespace TotalSynergyAPIService.Services
{
    public class ProjectService : IProjectService
    {
        private static readonly string apiURL = "https://beta.totalsynergy.com";
        private HttpClient _client { get; set; }

        public ProjectService(HttpClient client) {
            _client = client;
        }

        public async Task<PagedItemsResponse<Project>> GetProjects(string accessToken, string slug, int pageNumber = 1)
        {
            try
            {
                _client.DefaultRequestHeaders.Clear();
                _client.DefaultRequestHeaders.Add("access-token", accessToken);
                var response = await _client.GetAsync($"{apiURL}/api/v2/organisation/{slug}/Projects?includeCustomFields=true&page={pageNumber}&pageSize=10");

                if (!response.IsSuccessStatusCode)
                {
                    // An Error has occurred. Action appropriately
                    return null;
                }

                string rawResponse = await response.Content.ReadAsStringAsync();
                var pagedProject = JsonConvert.DeserializeObject<PagedItemsResponse<Project>>(rawResponse);

                return pagedProject;
            }
            catch (HttpRequestException e)
            {
                Console.WriteLine("\nException Caught!");
                Console.WriteLine("Message :{0} ", e.Message);
                //logFile("Exception: " + e.Message);
                return null;
            }
        }
    }
}
