﻿using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using System;
using System.Collections.Generic;
using System.IO;
using System.Net.Http;
using System.Threading.Tasks;
using System.Web;
using System.Windows;
using System.Windows.Controls;
using TotalSynergyAPIService.Domain.Models;
using TotalSynergyAPIService.Services;

namespace SynergyDemo
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        #region variable declarations
        String File_name = "Synergy Demo.log";

        private ProjectService _projectService { get; set; }
        private OrganisationService _organisationService { get; set; }
        private ProfileService _profileService { get; set; }

        #endregion variable declarations

        public MainWindow()
        {
            InitializeComponent();
            if (!File.Exists(File_name))
            {
                using (var myFile = File.Create(File_name))
                {
                    // interact with myFile here, it will be disposed automatically
                }
            }

            InitializeServices();
        }

        public void InitializeServices()
        {
            HttpClient client = new HttpClient();
            _projectService = new ProjectService(client);
            _organisationService = new OrganisationService(client);
            _profileService = new ProfileService(client);

        }

        #region API key change event
        //Text change event
        //Enable Request button 
        private void txtAPIKey_TextChanged(object sender, TextChangedEventArgs e)
        {
            if (string.IsNullOrWhiteSpace(this.txtAPIKey.Text))
            {
                btnRequest.IsEnabled = false;
            }
            else
            {
                btnRequest.IsEnabled = true;
            }
        }
        private void Hyperlink_RequestNavigate(object sender, EventArgs e)
        {
            System.Diagnostics.Process.Start("https://beta.totalsynergy.com/Profile/ApiKey");
        }
        #endregion API key change event

        #region Request button click event
        //Button click event
        //get the key and make API call
        private async void btnRequest_Click(object sender, RoutedEventArgs e)
        {
            // Clear response text block
            txtAPIResponseBlock.Text = "";

            string accessToken = txtAPIKey.Text;
            var profile = await GetProfile(accessToken);

            if (profile == null)
            {
                string message = "The key you copied from the profile page failed. Please carefully copy the whole key (it's pretty long), paste it in here, and try again";
                string title = "ERROR : Invalid key";
                MessageBox.Show(message, title);
                return;
            }

            // We have a valid profile, now let's get the first organisation associated with this
            // profile so we can get the slug.
            List<Organisation> organisations = await _organisationService.GetOrganisations(accessToken);

            if (organisations == null || organisations.Count == 0)
            {
                string message = "Could not find any organisations associated with this profile.";
                string title = "ERROR : No Organisation";
                MessageBox.Show(message, title);
                return;
            }

            logFile("Organisation Details:\n" + JsonConvert.SerializeObject(organisations));
            using (StreamWriter sw = new StreamWriter("Organisation.txt"))
            {
                sw.WriteLine(JsonConvert.SerializeObject(organisations));
            }

            // Now we have a slug to use via the organisation. Normally you would already know the slug 
            // you want to use for a API call.
            var projects = await _projectService.GetProjects(accessToken, organisations[0].slug);

            if (projects == null || projects.totalItems == 0)
            {
                string message = $"Could not find any projects associated with the organisation {organisations[0].slug}.";
                string title = "ERROR : No Projects";
                MessageBox.Show(message, title);
                return;
            }

            string json = JsonConvert.SerializeObject(projects, Formatting.Indented);
            JToken parsedJson = JToken.Parse(json);
            var beautified = parsedJson.ToString(Formatting.Indented);

            // Show json in response text block
            txtAPIResponseBlock.Text = beautified;

            return;
        }
        #endregion Request button click event

        #region Common functions

        private async Task<Profile> GetProfile(string accessToken)
        {
            var profile = await _profileService.GetProfile(accessToken);

            if (profile != null)
            {
                var profileJson = JsonConvert.SerializeObject(profile);
                logFile("Authentication successful");
                logFile("Profile Details:\n" + profileJson);

                using (StreamWriter sw = new StreamWriter("Profile.txt"))
                {
                    sw.WriteLine(JsonConvert.SerializeObject(profile));
                }
            }

            return profile;
        }

        private void logFile(string message, bool showDateTime = true)
        {
            string timeStamp = DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss tt");
            FileStream fs = new FileStream(File_name, FileMode.Append, FileAccess.Write);
            StreamWriter objWrite = new StreamWriter(fs);
            objWrite.Write((showDateTime == true ? ("\n" + timeStamp.ToString() + " | ") : "") + message);
            objWrite.Close();
        }
        #endregion Common
    }
}
