import { Component, Inject } from '@angular/core';
import { HttpClient } from '@angular/common/http';
import { ProfileService } from './fetch-data.service';
import { Profile } from './profile';

@Component({
  selector: 'app-fetch-data',
  templateUrl: './fetch-data.component.html'
})
export class FetchDataComponent {
  public accessToken: string;
  public contacts: string;
  private http: HttpClient;
  private baseUrl: string;
  private errorMessage = '';

  private getJsonFromUrl(url) {
    if (!url) url = location.href;
    var question = url.indexOf("?");
    var hash = url.indexOf("#");
    if (hash == -1 && question == -1) return {};
    if (hash == -1) hash = url.length;
    var query = question == -1 || hash == question + 1 ? url.substring(hash) :
      url.substring(question + 1, hash);
    var result = {};
    query.split("&").forEach(function (part) {
      if (!part) return;
      part = part.split("+").join(" "); // replace every + with space, regexp-free version
      var eq = part.indexOf("=");
      var key = eq > -1 ? part.substr(0, eq) : part;
      var val = eq > -1 ? decodeURIComponent(part.substr(eq + 1)) : "";
      var from = key.indexOf("[");
      if (from == -1) result[decodeURIComponent(key)] = val;
      else {
        var to = key.indexOf("]", from);
        var index = decodeURIComponent(key.substring(from + 1, to));
        key = decodeURIComponent(key.substring(0, from));
        if (!result[key]) result[key] = [];
        if (!index) result[key].push(val);
        else result[key][index] = val;
      }
    });
    return result;
  }


  constructor(http: HttpClient, @Inject('BASE_URL') baseUrl: string, private profileService: ProfileService) {
    this.http = http;
    this.baseUrl = baseUrl;
    console.log("fetch-data started")
    var fullLocation = window.location.href;
    console.log("full location:" + fullLocation);
    var jsonFromUrl = this.getJsonFromUrl(fullLocation)
    if (jsonFromUrl["code"] != undefined) {
      var requestCode = jsonFromUrl["code"]
      console.log("using code: " + requestCode);
      //convert code to access token
      http.get<string>(baseUrl + 'api/SampleData/GetAccessToken?requestCode=' + requestCode).subscribe(result => {
        this.accessToken = result;
        console.log("Access token " + this.accessToken)
      }, error => {
        console.log("error:" + error)
      }
      );
    }
  }

  // First example function is a proxy API example.
  public onGetProfile1() {
    console.log("getProfile1()")

    this.http.get<string>(this.baseUrl + 'api/SampleData/GetData?accessToken=' + this.accessToken).subscribe(result => {
      this.contacts = result;
      console.log("Access token " + this.accessToken)
    }, error => {
        this.contacts = "Error";
        console.log("error:" + error)
    });
  }

  // Second example function calls the API directly in javascript.
  public onGetProfile2() {
    this.profileService.getProfile(this.accessToken).subscribe(
      profile => this.contacts = JSON.stringify(profile),
      error => this.errorMessage = <any>error);
  }

  public documentExample() {

  }

}

