import { Component, Injectable } from '@angular/core';
import { HttpClient, HttpHeaders } from '@angular/common/http';

import { Observable, throwError } from 'rxjs';
import { catchError, tap } from 'rxjs/operators';

import { Profile } from './profile';

@Injectable({
  providedIn: 'root'
})
export class ProfileService {
  private profileUrl = 'https://beta.totalsynergy.com/api/v2/profile/index';

  constructor(private http: HttpClient) { }

  getProfile(accessToken): Observable<Profile> {

    let httpHeaders = new HttpHeaders()
      .set('access-token', accessToken);

    return this.http.get<Profile>(this.profileUrl, { headers: httpHeaders })
      .pipe(
        tap(data => console.log(JSON.stringify(data))),
        catchError(this.handleError)
      );
  }

  private handleError(err) {
    // in a real world app, we may send the server to some remote logging infrastructure
    // instead of just logging it to the console
    let errorMessage: string;
    if (err.error instanceof ErrorEvent) {
      // A client-side or network error occurred. Handle it accordingly.
      errorMessage = `An error occurred: ${err.error.message}`;
    }
    else if (err.error) {
      errorMessage = err.error;
    } else {
      // The backend returned an unsuccessful response code.
      // The response body may contain clues as to what went wrong,
      errorMessage = `Backend returned code ${err.status}: ${err.body.error}`;
    }
    console.error(err);
    return throwError(errorMessage);
  }

}
