using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Mvc;
using System.Net.Http;
using Newtonsoft.Json;
using ApiDemoApp.Models;
using Microsoft.AspNetCore.Http;
using System.IO;
using System.Text;
using Newtonsoft.Json.Linq;

namespace ApiDemoApp.Controllers
{
    [Route("api/[controller]")]
    public class SampleDataController : Controller
    {
        //private static readonly string apiURL = "http://localhost:5000";
        private static readonly string apiURL = "https://beta.totalsynergy.com";
        private static readonly HttpClient client = new HttpClient();

#if DEBUG
        public string applicationKey = "8388fb36-7074-41ba-bb6f-79f2cdb6d88d";
        public string applicationSecret = "18890fe0-a91d-4672-bea1-a301cc5aafca";
#else
        public string applicationKey = "8313fbfc-0f4b-4979-b225-b5187ca97263";
        public string applicationSecret = "378621c3-55f9-4fe5-960e-fa37bfa37433";
#endif

        [HttpGet("[action]")]
        public async Task<JsonResult> GetPublicKey()
        {
            return Json(applicationKey); 
        }

        [HttpGet("[action]")]
        public async Task<JsonResult> GetAccessToken(string requestCode)
        {

            var values = new Dictionary<string, string>
            {
               { "applicationKey", applicationKey },
               { "applicationSecret", applicationSecret },
               { "code", requestCode }
            };
            var content = new FormUrlEncodedContent(values);
            var response = await client.PostAsync($"{apiURL}/api/v2/Oauth2/GetAccessToken", content);
            var responseString = await response.Content.ReadAsStringAsync();
            var returnedValues = JsonConvert.DeserializeObject<Dictionary<string, string>>(responseString);
            return Json(returnedValues["accessToken"]);
        }

        [HttpGet("[action]")]
        public async Task<JsonResult> GetOrganisations(string accessToken)
        {
            HttpClient client = new HttpClient();
            client.DefaultRequestHeaders.Add("access-token", accessToken);
            var response = await client.GetAsync($"{apiURL}/api/v2/profile/GetActiveOrganisations");
            var responseString = await response.Content.ReadAsStringAsync();
            return Json(responseString);
        }

        [HttpGet("[action]")]
        public async Task<JsonResult> GetProjects(string slug, string accessToken)
        {
            HttpClient client = new HttpClient();
            client.DefaultRequestHeaders.Add("access-token", accessToken);
            var args = "page=0&pageSize=-1";
            var response = await client.GetAsync($"{apiURL}/api/v2/organisation/{slug}/projects/Index?{args}");
            var responseString = await response.Content.ReadAsStringAsync();
            
            return Json(responseString);
        }

        [HttpGet("[action]")]
        public async Task<JsonResult> GetData(string accessToken)
        {
            HttpClient client = new HttpClient();
            client.DefaultRequestHeaders.Add("access-token", accessToken);
            var response = await client.GetAsync($"{apiURL}/api/v2/profile/index");
            var responseString = await response.Content.ReadAsStringAsync();
            return Json(responseString);
        }

        [HttpGet("[action]")]
        public async Task<JsonResult> GetFolders(string slug, string projectId, string accessToken)
        {
            HttpClient client = new HttpClient();
            client.DefaultRequestHeaders.Add("access-token", accessToken);
            var response = await client.GetAsync($"{apiURL}/api/v2/organisation/{slug}/Documents/GetFolders?projectId={projectId}&contactId=null&staffId=null");
            var responseString = await response.Content.ReadAsStringAsync();

            return Json(responseString);
        }

        [HttpGet("[action]")]
        public async Task<JsonResult> GetDocuments(string slug, string projectId, string folderId, string accessToken)
        {
            HttpClient client = new HttpClient();
            client.DefaultRequestHeaders.Add("access-token", accessToken);
            var values = new Dictionary<string, string>
            {
               { "page", "0"},
               { "pageSize", "-1" },
                { "projectId", projectId},
                { "folderId", folderId}
            };
            var content = new FormUrlEncodedContent(values);
            
            var response = await client.PostAsync($"{apiURL}/api/v2/organisation/{slug}/Documents/GetPagedDocuments", content);
            var responseString = await response.Content.ReadAsStringAsync();

            return Json(responseString);
        }

        [HttpGet("[action]")]
        public async Task<FileResult> DownloadDocument(string slug, string id, string accessToken)
        {
            HttpClient client = new HttpClient();
            client.DefaultRequestHeaders.Add("access-token", accessToken);
            var response = await client.GetAsync($"{apiURL}/api/v2/organisation/{slug}/Documents/DownloadDocument?documentId={id}");
            var stream = await response.Content.ReadAsStreamAsync();
            return File(stream, "application/octet-stream");
        }

        [HttpPost("[action]")]
        public async Task<JsonResult> UploadDocument(string folderId, string projectId, string slug, string accessToken)
        {
            var uploadFile = HttpContext.Request.Form.Files[0];
            var fileName = uploadFile.FileName;
            var isFileDupliate = false;
            HttpClient client = new HttpClient();
            client.DefaultRequestHeaders.Add("access-token", accessToken);

            try
            {
                string[] uploadFileName = new string[] { uploadFile.FileName };

                var postFilesName = uploadFileName.ToArray();

                var postFilesNameString = JsonConvert.SerializeObject(postFilesName);
                var duplicateResponse = await client.PostAsync(
                    $"{apiURL}/api/v2/organisation/{slug}/Documents/GetDuplicates?projectId=" + projectId + "&folderId=" + folderId + "&contactId=null&staffId=null",
                    new StringContent(postFilesNameString, Encoding.UTF8, "application/json" ));

                var duplicateResult = await duplicateResponse.Content.ReadAsStringAsync();

                var duplicateJson = JObject.Parse(duplicateResult);
                var duplicateFiles = duplicateJson["data"];

                if (duplicateFiles.Count() > 0)
                {
                    fileName = fileName.Insert(fileName.LastIndexOf('.'), "(1)");
                    isFileDupliate = true;
                }

                MultipartFormDataContent form = new MultipartFormDataContent();
                HttpContent content = new StringContent("fileToUpload");
                HttpContent folderContent = new StringContent(folderId);
                HttpContent projectContent = new StringContent(projectId);
                HttpContent contactContent = new StringContent("");
                HttpContent staffContent = new StringContent("");
                HttpContent pathContent = new StringContent("");
                HttpContent conflictActionContent = new StringContent("");

                form.Add(content, "fileToUpload");
                form.Add(folderContent, "folderId");
                form.Add(projectContent, "projectId");
                form.Add(contactContent, "contactId");
                form.Add(staffContent, "staffId");
                form.Add(pathContent, "path");
                form.Add(conflictActionContent, "conflictAction");

                var stream = uploadFile.OpenReadStream();
                content = new StreamContent(stream);
                content.Headers.ContentDisposition = new System.Net.Http.Headers.ContentDispositionHeaderValue("form-data")
                {
                    Name = "fileToUpload",
                    FileName = fileName
                };

                form.Add(content);

                var response = await client.PostAsync($"{apiURL}/api/v2/organisation/{slug}/Documents/CreateDocument", form);
                response.Headers.Add("isFileDuplicate", isFileDupliate.ToString());
                response.Headers.Add("uploadFileName", fileName);
                return Json(response);
            }
            catch(Exception e)
            {
                return Json(e.Message);
            }

        }
    }
}
